unit dsdsound;
{*************TDSDMusic************
Extremely specialized DirectSound wrapper
Made in Delphi 4

Author:  Dag Sverre Seljebotn, dagsverre@sensewave.com
Created: 25-04-2000
History: 26-04-2000 First release

Needs DirectX 7.0 headers for Delphi. You can find them on www.delphi-jedi.org,
under the Graphics section (bottom-left...)

Created for the FF8 Configurator v. 2, http://ffsf.cjb.net

This code is distributed under the GNU General Public License.
NO WARRANTY. See COPYING for details.

Copyright (c) 2000 Dag Sverre Seljebotn
}

interface
uses Windows, MMSystem, Classes;

type
  TDSDSound = class
  private
    FDeviceList: TStringList;
    FHandle: Cardinal;
    function GetDeviceList: TStringList;
    function GetDeviceID(I: Integer): TGUID;
  public
    constructor Create(Handle: Cardinal);
    destructor Destroy; override;

    function IndexOfDevice(ID: TGUID): Integer;
    procedure PlaySound(Driver: Integer; Wave: TStream; Format: TWaveFormatEx;
      Pan: Integer; UseEAX: Boolean);

    property DeviceList: TStringList read GetDeviceList;
    property DeviceID[I: Integer]: TGUID read GetDeviceID;
  end;

implementation
uses ActiveX, ComObj, DirectSound, EAX;
{ TDSDSound }

constructor TDSDSound.Create(Handle: Cardinal);
begin
  inherited Create;
  FHandle:=Handle;
end;

destructor TDSDSound.Destroy;
var
  I: Integer;
begin
  for I:=0 to FDeviceList.Count - 1 do
    FreeMem(Pointer(FDeviceList.Objects[I]));
  FDeviceList.Free;
  inherited Destroy;
end;

function TDSDSound.GetDeviceID(I: Integer): TGUID;
begin
  Result:=PGUID(DeviceList.Objects[I])^;
end;

function EnumCallback(lpGUID: PGUID; lpDriverDescription: PAnsiChar;
      lpDriverName: PAnsiChar; lpContext: Pointer) : Longbool; stdcall;
const
  EmptyGUID: TGUID = '{00000000-0000-0000-0000-000000000000}';
var
  BufGUID: PGUID;
begin
  GetMem(BufGUID,SizeOf(TGUID));
  if lpGUID <> nil then
    BufGUID^:=lpGUID^
  else
    BufGUID^:=EmptyGUID;

  TStrings(lpContext).AddObject(lpDriverDescription, TObject(BufGUID));

  Result:=True;
end;

function TDSDSound.GetDeviceList: TStringList;
begin
  if FDeviceList = nil then
  begin
    FDeviceList:=TStringList.Create;
    DirectSoundEnumerate(@EnumCallback, FDeviceList);
  end;
  Result:=FDeviceList;
end;

function TDSDSound.IndexOfDevice(ID: TGUID): Integer;
var
  I: Integer;
begin
  Result:=-1;
  for I:=0 to FDeviceList.Count - 1 do
    if IsEqualGUID(PGUID(FDeviceList.Objects[I])^, ID) then
    begin
      Result:=I;
      Break;
    end;
end;


procedure TDSDSound.PlaySound(Driver: Integer; Wave: TStream; Format: TWaveFormatEx;
  Pan: Integer; UseEAX: Boolean);
var
  DirectSound: IDirectSound;
  SoundBuffer: IDirectSoundBuffer;

  Notify: TDSBPositionNotify;

  Caps: TDSBufferDesc;

  Buf1Ptr,
  Buf2Ptr: Pointer;
  Buf1Size,
  Buf2Size: Cardinal;

  GUID: TGUID;
begin
  //////////Init Directsound///////////
  GUID:=DeviceID[Driver];
  if UseEAX then
    EAXDirectSoundCreate(@GUID, DirectSound, nil)
  else
    DirectSoundCreate(@GUID, DirectSound, nil);

  OleCheck(DirectSound.SetCooperativeLevel(FHandle, DSSCL_PRIORITY));

  //////////Create buffer//////////////////
  FillChar(Caps,SizeOf(Caps),0);
  with Caps do
  begin
    dwSize:=SizeOf(Caps);
    dwBufferBytes:=Wave.Size;
    lpwfxFormat:=@Format;
    dwFlags:=DSBCAPS_CTRLPAN or DSBCAPS_STATIC or DSBCAPS_CTRLPOSITIONNOTIFY;
  end;
  OleCheck(DirectSound.CreateSoundBuffer(Caps, SoundBuffer, nil));

  ///////////////Write data to buffer/////////////
  OleCheck(SoundBuffer.Lock(0, Wave.Size, Buf1Ptr, Buf1Size, Buf2Ptr, Buf2Size, DSBLOCK_ENTIREBUFFER));
  try
    Wave.Read(Buf1Ptr^, Wave.Size);
  finally
    OleCheck(SoundBuffer.Unlock(Buf1Ptr, Buf1Size, Buf2Ptr, Buf2Size));
  end;

  ////////////////Set panning and effects///////
  OleCheck(SoundBuffer.SetPan(Pan));


  ///////////////Play buffer//////////////
  FillChar(Notify, SizeOf(Notify), 0);
  with Notify do
  begin
    dwOffset:=DSBPN_OFFSETSTOP;
    hEventNotify:=CreateEvent(nil, False, False, nil);
  end;
  try
    OleCheck((SoundBuffer as IDirectSoundNotify).SetNotificationPositions(1, Notify));
    OleCheck(SoundBuffer.Play(0, 0, 0));
    WaitForSingleObject(Notify.hEventNotify, INFINITE);
  finally
    CloseHandle(Notify.hEventNotify);
  end;
end;

end.
