unit gapfiles_bak;

{*************TMainForm************
Reader and writer class for .GAP files,
Game Addon Packs. Developed for use with Final Fantasy 8.
See http://sfff.cjb.net for more info.

Feel free to use this class if you do something similar with
another game.

The class structure was put toghether in a hurry, if you see
any improvements, please contect me:
dagsverre@sensewave.com

Made in Delphi 4.

Author:  Dag Sverre Seljebotn, dagsverre@sensewave.com
Created: 12-04-2000
History: //20-03-2000 First release

This code is distributed under the GNU General Public License.
NO WARRANTY. See COPYING for details.

Copyright (c) 2000 Dag Sverre Seljebotn
}


interface
  uses SysUtils, Classes;

type

{ Layout:

  1 x TGapHeader

  for each file:
  1 x TGapFile with data
  repeat...
}





  TCompressionMethod = (compNone, compZLib, compDiff,  compZLibDiff);

  TGapFileHeader = record
    HeaderSize,   //This is included so that backwards compatibility will be possible, should not include Data
    FileSize: Cardinal;
    Description: String[255];
    RelativePath: String[255];
    CompressMethod: TCompressionMethod;
    FileType,
    CRC,
    Flags: Cardinal;   //Currently only CRC in DIFF...
  end;

  PGapFileHeader = ^TGapFileHeader;

  TGapHeader = record
    HeaderSize,              //Again included for possibility of backwards compatibility
    Game: Cardinal;
    Description: String[255];
  end;


  TGapPackage = class
  private
    FFileStream: TFileStream;
    FFilename: String;
    FHeader: TGapHeader;
    FFileHeaders: TList;
    FFiles: TList;
    procedure Clear;
    function GetFileHeader(I: Integer): TGapFileHeader;
    function GetFileData(I: Integer): TStream;
  protected
    procedure AddFile(const FileData: TGapFileHeader; Data: TStream);
    property FileHeader[I: Integer]: TGapFileHeader read GetFileHeader;
    property FileData[I: Integer]: TStream read GetFileData;
    property Header: TGapHeader read FHeader write FHeader;
  public
    constructor Create(const AFilename: String; Mode: Word);
    destructor Destroy; override;
    property Filename: String read FFilename;
  end;

  TGapPackageWriter = class(TGapPackage)
  public
    constructor Create(const Filename: string);
    procedure AddFile(const ARelativePath, ADescription: String; Stream: TStream; StreamSize: Cardinal; ACompress: TCompressionMethod);
    procedure Write(GameCode: Cardinal; const ADescription: String);
  end;

  TGapPackageReader = class(TGapPackage)
  private

  public
    constructor Create(const Filename: string);
  end;


implementation

{ TGapFile }


procedure TGapPackage.AddFile(const FileData: TGapFileHeader; Data: TStream);
var
  GapFile: PGapFileHeader;
begin
  New(GapFile);
  GapFile^:=FileData;
  FFileHeaders.Add(GapFile);
  FFiles.Add(Data);
end;

procedure TGapPackage.Clear;
var
  I: Integer;
begin
  for I:=0 to FFileHeaders.Count - 1 do
    Dispose(FFileHeaders[I]);
  FFileHeaders.Clear;
  FFiles.Clear; //The calling code should dispose of it's TStreams
end;

constructor TGapPackage.Create(const AFilename: String; Mode: Word);
begin
  inherited Create;
  FFilename:=AFilename;
  FFileStream:=TFileStream.Create(AFilename, Mode);
  FFiles:=TList.Create;
  FFileHeaders:=TList.Create;
end;


destructor TGapPackage.Destroy;
begin
  Clear;
  FFileHeaders.Free;
  FFiles.Free;
end;

function TGapPackage.GetFileData(I: Integer): TStream;
begin
  Result:=TStream(FFiles[I]);
end;

function TGapPackage.GetFileHeader(I: Integer): TGapFileHeader;
begin
  Result:=PGapFileHeader(FFileHeaders[I])^;
end;


{ TGapPackageWriter }

procedure TGapPackageWriter.AddFile(const ARelativePath, ADescription: String; Stream: TStream;
  StreamSize: Cardinal; ACompress: TCompressionMethod);
var
  GapFile: TGapFileHeader;
begin
  with GapFile do
  begin
    HeaderSize:=SizeOf(GapFile);
    FileSize:=StreamSize;
    CompressMethod:=ACompress;
    RelativePath:=ARelativePath;
    Description:=ADescription;
  end;
  inherited AddFile(GapFile, Stream);
end;

constructor TGapPackageWriter.Create(const Filename: string);
begin
  inherited Create(Filename, fmCreate);
end;

procedure TGapPackageWriter.Write(GameCode: Cardinal; const ADescription: String);
var
  I: Integer;
  FileHeader: TGapFileHeader;
  Offset: Cardinal;
begin
  with Header do
  begin
    Game:=GameCode;
    Description:=ADescription;
  end;
  FFileStream.Write(Header, SizeOf(Header));
  for I:=0 to FFileHeaders.Count - 1 do
  begin
    FileHeader:=PGapFileHeader(FFileHeaders[I])^;
    FFileStream.WriteBuffer(FileHeader, FileHeader.HeaderSize);

    FFileStream.CopyFrom(TStream(FFiles[I]), FileHeader.FileSize);
  end;
end;

{ TGapPackageReader }

constructor TGapPackageReader.Create(const Filename: string);
begin
  inherited Create(Filename, fmOpenRead or fmShareDenyNone);
end;

end.
