unit sfman;

interface
  uses windows;


const
  SFMANCAP_DYNAMIC_MEM_CAP            = $80000000;
  SFMANCAP_SOFTWARE_SYNTH_CAP         = $40000000;
  SFMANCAP_NONMMSYSTEM_COMPATIBLE_CAP = $20000000;
  SFMANCAP_NONWAVEFORM_SUPPORT_CAP    = $10000000;

  ID_SFMANL100API                     = $00010000;
  ID_SFMANL101API                     = $00010001;

  SFFLAG_OPER_FILE                    = 0;
  SFFLAG_OPER_MEMORY                  = 1;
type
  TSFBankStyle = (SFBANK_AWE32, SFBANK_MMA);


  TSFMidiLocation = packed record
    m_BankIndex    : Word;
    m_PresetIndex  : Word;
  end;
  PSFMidiLocation = ^TSFMidiLocation;

  TSFBufferObject = packed record
    m_Size         : Word;         //* The size of the buffer attached */
    m_Buffer       : PChar;        //* The pointer to the buffer itself */
    m_SizeUsed     : LongWord;     //* The number of buffer characters used by DLL */
    m_Flag         : Word;         //* Scratch Variable */
  end;
  PSFBufferObject = ^TSFBufferObject;

  TSFCaps = packed record
    m_SizeOf       : Word;                   //* sizeof this structure                    */
    m_RouterID     : LongWord;               //* SF Device Router ID                      */
    m_DevCaps      : LongWord;               //* Capabilities                             */
    m_MoreDevCaps  : LongWord;               //* More capabilities                        */
    m_RomID        : LongWord;               //* ID for a Sound ROM used in a device      */
    m_RomVer       : LongWord;               //* Version for a Sound ROM used in a device */
    m_DevName      : array[1..40] of Char;   //* MMSystem name for MIDI Output Device     */
    m_DevMgrName   : array[1..20] of Char;   //* File Name of the Device Manager DLL.     */
    m_DevMgrEntry  : array[1..20] of Char;   //* Entry point inside the Device Manager DLL*/
  end;

  LResult = LongWord;

  TSF101VTable = packed record
    // Global functions
    GetNumDevs                  : function (var Count: Word): LResult; cdecl;

    // Device functions which do not require allocation (opening)
    GetDevCaps                  : function (DevIndex: Word; var Caps: TSFCaps): LResult; cdecl;
    GetRouterID                 : function (DevIndex: Word; var ID: Word): LResult; cdecl;
    IsDeviceFree                : function (DevIndex: Word): LResult; cdecl;
    Open                        : function (DevIndex: Word): LResult; cdecl;
    Close                       : function (DevIndex: Word): LResult; cdecl;

    // Query functions
    IsMIDIBankUsed              : function (DevIndex: Word; var Param: Word): LResult; cdecl;
    QueryStaticSampleMemorySize : function (DevIndex: Word; var P1, P2: Word): LResult; cdecl;

    // Synthesizer Emulation Functions
    GetAllSynthEmulations       : function (DevIndex: Word; var Buffer: TSFBufferObject): LResult; cdecl;
    GetSynthEmulation           : function (DevIndex: Word; var Buffer: TSFBufferObject): LResult; cdecl;
    SelectSynthEmulation        : function (DevIndex: Word; Buffer: Word): LResult; cdecl;

    // Sound Bank management functions
    LoadBank                    : function (DevIndex: Word; Location: PSFMidiLocation; Buffer: PSFBufferObject): LResult; cdecl;
    GetLoadedBankDescriptor     : function (DevIndex: Word; var Location: TSFMidiLocation; var Buffer: TSFBufferObject): LResult; cdecl;
    GetLoadedBankPathname       : function (DevIndex: Word; var Location: TSFMidiLocation; var Buffer: TSFBufferObject): LResult; cdecl;
    ClearLoadedBank             : function (DevIndex: Word; var Location: TSFMidiLocation): LResult; cdecl;

    // Sound Preset management functions
    LoadPreset                  : function (DevIndex: Word; var Location, Location2: TSFMidiLocation; var Buffer: TSFBufferObject): LResult; cdecl;
    GetLoadedPresetDescriptor   : function (DevIndex: Word; var Location: TSFMidiLocation; var Buffer: TSFBufferObject): LResult; cdecl;
    ClearLoadedPreset           : function (DevIndex: Word; var Location: TSFMidiLocation): LResult; cdecl;

    // Sound Waveform management functions
    LoadWaveForm                : function (DevIndex: Word; var Location: TSFMidiLocation; var Buffer: TSFBufferObject): LResult; cdecl;
    ClearLoadedWaveform         : function (DevIndex: Word; var Location: TSFMidiLocation): LResult; cdecl;
    DeviceSpecificMessage       : function (DevIndex: Word; Msg, WParam, LParam: Cardinal): LResult; cdecl;
  end;
  PSF101VTable = ^TSF101VTable;


  TSFManager = packed record
    QueryInterface: function (InterfaceID: Cardinal; var VTable: PSF101VTable): LResult; cdecl;
  end;
  PSFManager = ^TSFManager;

var
  SFManager: TSF101VTable;

implementation
var
  DLLHandle: THandle;

procedure LoadDLL;
var
  VTable: PSF101VTable;

begin
  DLLHandle:=LoadLibrary('sfman32.dll');
  PSFManager(GetProcAddress(DllHandle, 'SFManager'))^.QueryInterface(ID_SFMANL101API, VTable);
  SFManager:=VTable^;
end;

procedure FreeDLL;
begin
  FreeLibrary(DllHandle);
end;

initialization
  LoadDLL;
finalization
  FreeDLL;

end.
