unit rar;

interface
  uses Windows;

type
  TChangeVolProc      = function (var ArcName : PChar; Mode : Integer) : Integer; cdecl;
  TProcessDataProc    = function (Addr : PChar; Size : Integer) : Integer; cdecl;
  TNewFileProc        = procedure (Filename:string);

function ListRar(ArchiveName:string;var List:array of string;var size:integer):integer;

function ExtractRar(const ArchiveName,TargetDir:string;Filenames: array of String;ChangeVolume:TChangeVolProc;ProcessData:TProcessDataProc;NewFile:TNewFileProc;const Password:string):Boolean;
function LoadDll(Filename:string):boolean;
procedure FreeDll;

const
  rMaxCommentSize = 65535; {Modify this to change the limit of Comment size}

  erEndArchive    = 10; {End of archive}
  erNoMemory      = 11; {Not enough memory to initialize data structures}
  erBadData       = 12; {Archive header broken}
  erBadArchive    = 13; {File is not valid RAR archive}
  erUnknownFormat = 14; {UnKnown comment format}
  erEOpen         = 15; {File open error}
  erECreate       = 16; {File create error}
  erEClose        = 17; {File close error}
  erERead         = 18; {Read error}
  erEWrite        = 19; {Write error}
  erSmallBuf      = 20; {Buffer too small, comments weren't read completely}

  opList          =  0; {Open archive for reading file headers only}
  opExtract       =  1; {Open archive for testing and extracting files}

  doSkip          =  0; {Move to the next file in archive}
                        {Warning: If the archive is solid and opExtract mode                                  was set when the archive was opened, the
                                  current file will be processed - the operation
                                  will be performed slower than a simple seek}
  doTest          =  1; {Test the current file and move to the next file in
                         the archive. If the archive was opened with opList mode,
                         the operation is equal to doSkip}
  doExtract       =  2; {Extract the current file and move to the next file.
                         If the archive was opened with opList mode,
                         the operation is equal to doSkip}

  moVolAsk        =  0; {Required volume is absent. The function should prompt
                         user and return non-zero value to retry the operation.
                         The function may also specify a new volume name,
                         placing it to ArcName parameter}
                         {!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}
  moVolNotify     =  1; {Required volume is successfully opened. This is a
                         notification call and ArcName modification is NOT
                         allowed. The funciton should return non-zero value
                         to continue or a zero value to terminate operation}
                         {!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}


type

    THandle=integer;

    RARHeaderData = record
                    ArcName                            : array[1..260] of char;
                    FileName                           : array[1..260] of char;
                    Flags                              : Cardinal;
                    PackSize                           : Cardinal;
                    UnpSize                            : Cardinal;
                    HostOS                             : Cardinal;
                    FileCRC                            : Cardinal;
                    FileTime                           : Cardinal;
                    UnpVer                             : Cardinal;
                    Method                             : Cardinal;
                    FileAttr                           : Cardinal;
                    CmtBuf                             : PChar;
                    CmtBufSize, CmtSize, CmtState      : Cardinal;
                  end;

  RAROpenArchiveData = record
                         ArcName                       : PChar;
                         OpenMode                      : Cardinal;
                         OpenResult                    : Cardinal;
                         CmtBuf                        : PChar;
                         CmtBufSize                    : Cardinal;
                         CmtSize                       : Cardinal;
                         CmtState                      : Cardinal;
                       end;

  TComment = record
               Size : Integer;
               Data : Array[1..rMaxCommentSize] of Char;
             end;

//  TChangeVolProc      = function (var ArcName : PChar; Mode : Integer) : Integer; cdecl;
//  TProcessDataProc    = function (Addr : PChar; Size : Integer) : Integer; cdecl;

var
  RAROpenArchive  : function (var ArchiveData : RAROpenArchiveData) : THandle; stdcall;
  RARCloseArchive : function (hArcData : THandle) : Integer; stdcall;
  RARReadHeader   : function (hArcData : THandle; var HeaderData : RARHeaderData) : Integer; stdcall;
  RARProcessFile  : function (hArcData : THandle; Operation : Integer; DestPath, DestName : PChar) : Integer; stdcall;
  RARDllHandle: integer;

  RARSetChangeVolProc   : procedure (hArcData : THandle; CVP : TChangeVolProc); stdcall;
  RARSetProcessDataProc : procedure (hArcData : THandle; PDP : TProcessDataProc); stdcall;
  RARSetPassword        : procedure (hArcData : THandle; Password : PChar); stdcall;

implementation

function LoadDll(Filename:string):boolean;
begin
   RarDllHandle := LoadLibrary(PChar(Filename));
   if RarDllHandle > HInstance_Error then
   begin
     @RAROpenArchive  :=GetProcAddress(RARDllHandle,'RAROpenArchive');
     @RARCloseArchive :=GetProcAddress(RARDllHandle,'RARCloseArchive');
     @RARReadHeader   :=GetProcAddress(RARDllHandle,'RARReadHeader');
     @RARProcessFile  :=GetProcAddress(RARDllHandle,'RARProcessFile');
     @RARSetChangeVolProc   :=GetProcAddress(RARDllHandle,'RARSetChangeVolProc');
     @RARSetProcessDataProc :=GetProcAddress(RARDllHandle,'RARSetProcessDataProc');
     @RARSetPassword        :=GetProcAddress(RARDllHandle,'RARSetPassword');
     result:=true;
   end
   else
   begin
     RARDllHandle := 0; {reset}
     result:=false;
   end;
end;

procedure FreeDll;
begin
  FreeLibrary(RARDllHandle);
end;

function LowerCase(const S: string): string;
var
  Ch: Char;
  L: Integer;
  Source, Dest: PChar;
begin
  L := Length(S);
  SetLength(Result, L);
  Source := Pointer(S);
  Dest := Pointer(Result);
  while L <> 0 do
  begin
    Ch := Source^;
    if (Ch >= 'A') and (Ch <= 'Z') then Inc(Ch, 32);
    Dest^ := Ch;
    Inc(Source);
    Inc(Dest);
    Dec(L);
  end;
end;

function IsInArray(s:string;a:array of string):boolean;
var
  i:integer;
begin
  result:=false;
  for i:=low(a) to high(a) do
  begin
    if lowercase(s)=lowercase(a[i]) then
    begin
      result:=true;
      break;
    end;
  end;
end;

function ExtractRar(const ArchiveName,TargetDir:string;Filenames: array of String;ChangeVolume:TChangeVolProc;ProcessData:TProcessDataProc;NewFile:TNewFileProc;const Password:string):Boolean;
var OpenArchiveData : RAROpenArchiveData;
    hArcData        : THandle;
    RHCode, PFCode  : Integer;
    HeaderData      : RARHeaderData;
    Comm            : TComment;
    TDir            : PChar;
    Temp            : Integer;
    S               : String;
    Filesdone       : Integer;
begin
  Result:=True;
  OpenArchiveData.ArcName:=PChar(ArchiveName);
  {!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}
  OpenArchiveData.CmtBuf:=@Comm.Data[1];
  OpenArchiveData.CmtBufSize:=SizeOf(Comm.Data);
  OpenArchiveData.OpenMode:=opExtract;
  hArcData:=RAROpenArchive(OpenArchiveData);

  try
    if Assigned(ChangeVolume) then
      RARSetChangeVolProc(hArcData,ChangeVolume);
    if Assigned(ProcessData) then
      RARSetProcessDataProc(hArcData,ProcessData);

    if Password<>'' then
      RARSetPassword(hArcData,PChar(Password));

    Comm.Size:=OpenArchiveData.CmtSize;

    If TargetDir='' then TDir:=Nil else TDir:=@TargetDir[1];

    if OpenArchiveData.OpenResult<>0 then
    begin
      Result:=False;
      Exit;
    end;
    FilesDone:=0;
    HeaderData.CmtBuf:=@Comm.Data[1];
    HeaderData.CmtBufSize:=SizeOf(Comm.Data);
    RHCode:=RARReadHeader(hArcData,HeaderData);
    while RHCode=0 do
    begin
      S:='';
      for Temp:=1 to SizeOf(HeaderData.FileName) do
        if HeaderData.FileName[Temp]=#0 then
          break
        else
          S:=S+HeaderData.FileName[Temp];

      if (High(Filenames)=-1) or (Filenames[0]='') or IsInArray(S,Filenames) then
      begin
        if Assigned(NewFile) then
          NewFile(s);
        PFCode:=RARProcessFile(hArcData,doExtract,TDir,NiL);
        if PFCode<>0 then
        begin
          Result:=False;
          if (PFCode<>erECreate) and (PFCode<>erBadData) then
            Exit;
        end;
        inc(FilesDone);
        if (Filenames[0]<>'')and(Filesdone>High(Filenames)-1) then
          exit;
      end
      else
      begin
        PFCode:=RARProcessFile(hArcData,doSkip,TDir,NiL);
        if PFCode<>0 then
        begin
          Result:=False;
          Exit;
        end;
      end;
      RHCode:=RARReadHeader(hArcData,HeaderData);
    end;
  finally
    RARCloseArchive(hArcData);
  end;
end;


function ListRar(ArchiveName:string;var List:array of string;var size:integer):integer;
var OpenArchiveData : RAROpenArchiveData;
    hArcData        : THandle;
    RHCode, PFCode  : Integer;
    HeaderData      : RARHeaderData;
    Comm            : TComment;
    Temp            : Integer;
    S               : String;
begin
  Result:=0;
  Size:=0;
  OpenArchiveData.ArcName:=@ArchiveName[1];
  {!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}
  OpenArchiveData.CmtBuf:=@Comm.Data[1];
  OpenArchiveData.CmtBufSize:=SizeOf(Comm.Data);
  OpenArchiveData.OpenMode:=opList;
  hArcData:=RAROpenArchive(OpenArchiveData);

//  if Password<>'' then
//    RARSetPassword(hArcData,PChar(Password));

  Comm.Size:=OpenArchiveData.CmtSize;

  if OpenArchiveData.OpenResult<>0 then
  begin
    RARCloseArchive(hArcData);
    Result:=-1;
    Exit;
  end;

  HeaderData.CmtBuf:=@Comm.Data[1];
  HeaderData.CmtBufSize:=SizeOf(Comm.Data);
  RHCode:=RARReadHeader(hArcData,HeaderData);
  while RHCode=0 do
  begin
    inc(Size,HeaderData.UnpSize);
    S:='';
    for Temp:=1 to SizeOf(HeaderData.FileName) do
      if HeaderData.FileName[Temp]=#0 then
        break
      else
        S:=S+HeaderData.FileName[Temp];


    PFCode:=RARProcessFile(hArcData,doSkip,'',NiL);
    if PFCode<>0 then
       begin
         Result:=-1;
         RARCloseArchive(hArcData);
         Exit;
       end;

    List[result]:=S;
    inc(Result);
    RHCode:=RARReadHeader(hArcData,HeaderData);
  end;

  RARCloseArchive(hArcData);
end;


end.
